@@ -9,10 +9,10 @@ import pandas as pd
 import statsmodels.api as sm
 from scipy import stats
 from sklearn.impute import SimpleImputer
-    
+
+
 class FilterSelect:
-    """A class for feature importance methods.
-    """
+    """A class for feature importance methods."""
 
     def __init__(self):
         return
@@ -35,23 +35,29 @@ class FilterSelect:
         Y = data[y_name]
         X = data[[treatment_indicator, feature_name]]
         X = sm.add_constant(X)
-        X['{}-{}'.format(treatment_indicator, feature_name)] = X[[treatment_indicator, feature_name]].product(axis=1)
+        X["{}-{}".format(treatment_indicator, feature_name)] = X[
+            [treatment_indicator, feature_name]
+        ].product(axis=1)
 
         model = sm.OLS(Y, X)
         result = model.fit()
 
         F_test = result.f_test(np.array([0, 0, 0, 1]))
-        F_test_result = pd.DataFrame({
-            'feature': feature_name, # for the interaction, not the main effect
-            'method': 'F-statistic',
-            'score': F_test.fvalue[0][0],
-            'p_value': F_test.pvalue,
-            'misc': 'df_num: {}, df_denom: {}'.format(F_test.df_num, F_test.df_denom),
-        }, index=[0]).reset_index(drop=True)
+        F_test_result = pd.DataFrame(
+            {
+                "feature": feature_name,  # for the interaction, not the main effect
+                "method": "F-statistic",
+                "score": F_test.fvalue[0][0],
+                "p_value": F_test.pvalue,
+                "misc": "df_num: {}, df_denom: {}".format(
+                    F_test.df_num, F_test.df_denom
+                ),
+            },
+            index=[0],
+        ).reset_index(drop=True)
 
         return F_test_result
 
-
     def filter_F(self, data, treatment_indicator, features, y_name):
         """
         Rank features based on the F-statistics of the interaction.
@@ -68,19 +74,23 @@ class FilterSelect:
         """
         all_result = pd.DataFrame()
         for x_name_i in features:
-            one_result = self._filter_F_one_feature(data=data,
-                treatment_indicator=treatment_indicator, feature_name=x_name_i, y_name=y_name
+            one_result = self._filter_F_one_feature(
+                data=data,
+                treatment_indicator=treatment_indicator,
+                feature_name=x_name_i,
+                y_name=y_name,
             )
             all_result = pd.concat([all_result, one_result])
 
-        all_result = all_result.sort_values(by='score', ascending=False)
-        all_result['rank'] = all_result['score'].rank(ascending=False)
+        all_result = all_result.sort_values(by="score", ascending=False)
+        all_result["rank"] = all_result["score"].rank(ascending=False)
 
         return all_result
 
-
     @staticmethod
-    def _filter_LR_one_feature(data, treatment_indicator, feature_name, y_name, disp=True):
+    def _filter_LR_one_feature(
+        data, treatment_indicator, feature_name, y_name, disp=True
+    ):
         """
         Conduct LR (Likelihood Ratio) test of the interaction between treatment and one feature.
 
@@ -104,7 +114,9 @@ class FilterSelect:
 
         # Full model (with interaction)
         X_f = X_r.copy()
-        X_f['{}-{}'.format(treatment_indicator, feature_name)] = X_f[[treatment_indicator, feature_name]].product(axis=1)
+        X_f["{}-{}".format(treatment_indicator, feature_name)] = X_f[
+            [treatment_indicator, feature_name]
+        ].product(axis=1)
         model_f = sm.Logit(Y, X_f)
         result_f = model_f.fit(disp=disp)
 
@@ -112,17 +124,19 @@ class FilterSelect:
         LR_df = len(result_f.params) - len(result_r.params)
         LR_pvalue = 1 - stats.chi2.cdf(LR_stat, df=LR_df)
 
-        LR_test_result = pd.DataFrame({
-            'feature': feature_name, # for the interaction, not the main effect
-            'method': 'LRT-statistic',
-            'score': LR_stat,
-            'p_value': LR_pvalue,
-            'misc': 'df: {}'.format(LR_df),
-        }, index=[0]).reset_index(drop=True)
+        LR_test_result = pd.DataFrame(
+            {
+                "feature": feature_name,  # for the interaction, not the main effect
+                "method": "LRT-statistic",
+                "score": LR_stat,
+                "p_value": LR_pvalue,
+                "misc": "df: {}".format(LR_df),
+            },
+            index=[0],
+        ).reset_index(drop=True)
 
         return LR_test_result
 
-
     def filter_LR(self, data, treatment_indicator, features, y_name, disp=True):
         """
         Rank features based on the LRT-statistics of the interaction.
@@ -139,22 +153,28 @@ class FilterSelect:
         """
         all_result = pd.DataFrame()
         for x_name_i in features:
-            one_result = self._filter_LR_one_feature(data=data,
-                treatment_indicator=treatment_indicator, feature_name=x_name_i, y_name=y_name, disp=disp
+            one_result = self._filter_LR_one_feature(
+                data=data,
+                treatment_indicator=treatment_indicator,
+                feature_name=x_name_i,
+                y_name=y_name,
+                disp=disp,
             )
             all_result = pd.concat([all_result, one_result])
 
-        all_result = all_result.sort_values(by='score', ascending=False)
-        all_result['rank'] = all_result['score'].rank(ascending=False)
+        all_result = all_result.sort_values(by="score", ascending=False)
+        all_result["rank"] = all_result["score"].rank(ascending=False)
 
         return all_result
 
-
     # Get node summary - a function
     @staticmethod
-    def _GetNodeSummary(data,
-                        experiment_group_column='treatment_group_key',
-                        y_name='conversion', smooth=True):
+    def _GetNodeSummary(
+        data,
+        experiment_group_column="treatment_group_key",
+        y_name="conversion",
+        smooth=True,
+    ):
         """
         To count the conversions and get the probabilities by treatment groups. This function comes from the uplift tree algorithm, that is used for tree node split evaluation.
 
@@ -196,9 +216,13 @@ class FilterSelect:
             results.update({ti: {}})
             for ci in y_name_keys:
                 if smooth:
-                    results[ti].update({ci: results_series[ti, ci]
-                                        if results_series.index.isin([(ti, ci)]).any()
-                                        else 1})
+                    results[ti].update(
+                        {
+                            ci: results_series[ti, ci]
+                            if results_series.index.isin([(ti, ci)]).any()
+                            else 1
+                        }
+                    )
                 else:
                     results[ti].update({ci: results_series[ti, ci]})
 
@@ -206,8 +230,9 @@ class FilterSelect:
         nodeSummary = {}
         for treatment_group_key in results:
             n_1 = results[treatment_group_key].get(1, 0)
-            n_total = (results[treatment_group_key].get(1, 0)
-                       + results[treatment_group_key].get(0, 0))
+            n_total = results[treatment_group_key].get(1, 0) + results[
+                treatment_group_key
+            ].get(0, 0)
             y_mean = 1.0 * n_1 / n_total
             nodeSummary[treatment_group_key] = [y_mean, n_total]
 
@@ -227,10 +252,10 @@ class FilterSelect:
             qk = 0.1**6
         elif qk > 1 - 0.1**6:
             qk = 1 - 0.1**6
-        S = pk * np.log(pk / qk) + (1-pk) * np.log((1-pk) / (1-qk))
+        S = pk * np.log(pk / qk) + (1 - pk) * np.log((1 - pk) / (1 - qk))
         return S
 
-    def _evaluate_KL(self, nodeSummary, control_group='control'):
+    def _evaluate_KL(self, nodeSummary, control_group="control"):
         """
         Calculate the multi-treatment unconditional D (one node)
         with KL Divergence as split Evaluation function.
@@ -253,7 +278,7 @@ class FilterSelect:
         return d_res
 
     @staticmethod
-    def _evaluate_ED(nodeSummary, control_group='control'):
+    def _evaluate_ED(nodeSummary, control_group="control"):
         """
         Calculate the multi-treatment unconditional D (one node)
         with Euclidean Distance as split Evaluation function.
@@ -268,11 +293,11 @@ class FilterSelect:
         d_res = 0
         for treatment_group in nodeSummary:
             if treatment_group != control_group:
-                d_res += 2 * (nodeSummary[treatment_group][0] - pc)**2
+                d_res += 2 * (nodeSummary[treatment_group][0] - pc) ** 2
         return d_res
 
     @staticmethod
-    def _evaluate_Chi(nodeSummary, control_group='control'):
+    def _evaluate_Chi(nodeSummary, control_group="control"):
         """
         Calculate the multi-treatment unconditional D (one node)
         with Chi-Square as split Evaluation function.
@@ -287,17 +312,22 @@ class FilterSelect:
         d_res = 0
         for treatment_group in nodeSummary:
             if treatment_group != control_group:
-                d_res += (
-                    (nodeSummary[treatment_group][0] - pc)**2 / max(0.1**6, pc)
-                    + (nodeSummary[treatment_group][0] - pc)**2 / max(0.1**6, 1-pc)
-                )
+                d_res += (nodeSummary[treatment_group][0] - pc) ** 2 / max(
+                    0.1**6, pc
+                ) + (nodeSummary[treatment_group][0] - pc) ** 2 / max(0.1**6, 1 - pc)
         return d_res
 
-
-    def _filter_D_one_feature(self, data, feature_name, y_name,
-                              n_bins=10, method='KL', control_group='control',
-                              experiment_group_column='treatment_group_key',
-                              null_impute=None):
+    def _filter_D_one_feature(
+        self,
+        data,
+        feature_name,
+        y_name,
+        n_bins=10,
+        method="KL",
+        control_group="control",
+        experiment_group_column="treatment_group_key",
+        null_impute=None,
+    ):
         """
         Calculate the chosen divergence measure for one feature.
 
@@ -322,59 +352,77 @@ class FilterSelect:
         """
         # [TODO] Application to categorical features
 
-        if method == 'KL':
+        if method == "KL":
             evaluationFunction = self._evaluate_KL
-        elif method == 'ED':
+        elif method == "ED":
             evaluationFunction = self._evaluate_ED
-        elif method == 'Chi':
+        elif method == "Chi":
             evaluationFunction = self._evaluate_Chi
 
         totalSize = len(data.index)
 
         # impute null if enabled
         if null_impute is not None:
-            data[feature_name] = SimpleImputer(missing_values=np.nan, strategy=null_impute).fit_transform(data[feature_name].values.reshape(-1, 1))
+            data[feature_name] = SimpleImputer(
+                missing_values=np.nan, strategy=null_impute
+            ).fit_transform(data[feature_name].values.reshape(-1, 1))
         elif data[feature_name].isna().any():
-            raise Exception("Null value(s) present in column '{}'. Please impute the null value or use null_impute parameter provided!!!".format(feature_name))
+            raise Exception(
+                "Null value(s) present in column '{}'. Please impute the null value or use null_impute parameter provided!!!".format(
+                    feature_name
+                )
+            )
 
         # drop duplicate edges in pq.cut result to avoid issues
-        x_bin = pd.qcut(data[feature_name].values, n_bins, labels=False,
-                        duplicates='drop')
+        x_bin = pd.qcut(
+            data[feature_name].values, n_bins, labels=False, duplicates="drop"
+        )
 
         d_children = 0
 
-        for i_bin in range(np.nanmax(x_bin).astype(int) + 1): # range(n_bins):
+        for i_bin in range(np.nanmax(x_bin).astype(int) + 1):  # range(n_bins):
             nodeSummary = self._GetNodeSummary(
                 data=data.loc[x_bin == i_bin],
-                experiment_group_column=experiment_group_column, y_name=y_name
+                experiment_group_column=experiment_group_column,
+                y_name=y_name,
             )[1]
-            nodeScore = evaluationFunction(nodeSummary,
-                                           control_group=control_group)
+            nodeScore = evaluationFunction(nodeSummary, control_group=control_group)
             nodeSize = sum([x[1] for x in list(nodeSummary.values())])
             d_children += nodeScore * nodeSize / totalSize
 
         parentNodeSummary = self._GetNodeSummary(
             data=data, experiment_group_column=experiment_group_column, y_name=y_name
         )[1]
-        d_parent = evaluationFunction(parentNodeSummary,
-                                      control_group=control_group)
+        d_parent = evaluationFunction(parentNodeSummary, control_group=control_group)
 
         d_res = d_children - d_parent
 
-        D_result = pd.DataFrame({
-            'feature': feature_name,
-            'method': method,
-            'score': d_res,
-            'p_value': None,
-            'misc': 'number_of_bins: {}'.format(min(n_bins, np.nanmax(x_bin).astype(int) + 1)),# format(n_bins),
-        }, index=[0]).reset_index(drop=True)
+        D_result = pd.DataFrame(
+            {
+                "feature": feature_name,
+                "method": method,
+                "score": d_res,
+                "p_value": None,
+                "misc": "number_of_bins: {}".format(
+                    min(n_bins, np.nanmax(x_bin).astype(int) + 1)
+                ),  # format(n_bins),
+            },
+            index=[0],
+        ).reset_index(drop=True)
 
-        return(D_result)
+        return D_result
 
-    def filter_D(self, data, features, y_name,
-                 n_bins=10, method='KL', control_group='control',
-                 experiment_group_column='treatment_group_key',
-                 null_impute=None):
+    def filter_D(
+        self,
+        data,
+        features,
+        y_name,
+        n_bins=10,
+        method="KL",
+        control_group="control",
+        experiment_group_column="treatment_group_key",
+        null_impute=None,
+    ):
         """
         Rank features based on the chosen divergence measure.
 
@@ -402,25 +450,34 @@ class FilterSelect:
 
         for x_name_i in features:
             one_result = self._filter_D_one_feature(
-                data=data, feature_name=x_name_i, y_name=y_name,
-                n_bins=n_bins, method=method, control_group=control_group,
+                data=data,
+                feature_name=x_name_i,
+                y_name=y_name,
+                n_bins=n_bins,
+                method=method,
+                control_group=control_group,
                 experiment_group_column=experiment_group_column,
-                null_impute=null_impute
+                null_impute=null_impute,
             )
             all_result = pd.concat([all_result, one_result])
 
-        all_result = all_result.sort_values(by='score', ascending=False)
-        all_result['rank'] = all_result['score'].rank(ascending=False)
+        all_result = all_result.sort_values(by="score", ascending=False)
+        all_result["rank"] = all_result["score"].rank(ascending=False)
 
         return all_result
 
-    def get_importance(self, data, features, y_name, method,
-                      experiment_group_column='treatment_group_key',
-                      control_group = 'control',
-                      treatment_group = 'treatment',
-                      n_bins=5,
-                      null_impute=None
-                      ):
+    def get_importance(
+        self,
+        data,
+        features,
+        y_name,
+        method,
+        experiment_group_column="treatment_group_key",
+        control_group="control",
+        treatment_group="treatment",
+        n_bins=5,
+        null_impute=None,
+    ):
         """
         Rank features based on the chosen statistic of the interaction.
 
@@ -443,28 +500,47 @@ class FilterSelect:
             all_result : pd.DataFrame
                 a data frame with following columns: ['method', 'feature', 'rank', 'score', 'p_value', 'misc']
         """
-        
-        if method == 'F':
-            data = data[data[experiment_group_column].isin([control_group, treatment_group])]
-            data['treatment_indicator'] = 0
-            data.loc[data[experiment_group_column]==treatment_group,'treatment_indicator'] = 1
-            all_result = self.filter_F(data=data, 
-                treatment_indicator='treatment_indicator', features=features, y_name=y_name
+
+        if method == "F":
+            data = data[
+                data[experiment_group_column].isin([control_group, treatment_group])
+            ]
+            data["treatment_indicator"] = 0
+            data.loc[
+                data[experiment_group_column] == treatment_group, "treatment_indicator"
+            ] = 1
+            all_result = self.filter_F(
+                data=data,
+                treatment_indicator="treatment_indicator",
+                features=features,
+                y_name=y_name,
             )
-        elif method == 'LR':
-            data = data[data[experiment_group_column].isin([control_group, treatment_group])]
-            data['treatment_indicator'] = 0
-            data.loc[data[experiment_group_column]==treatment_group,'treatment_indicator'] = 1
-            all_result = self.filter_LR(data=data, disp=True,
-                treatment_indicator='treatment_indicator', features=features, y_name=y_name
+        elif method == "LR":
+            data = data[
+                data[experiment_group_column].isin([control_group, treatment_group])
+            ]
+            data["treatment_indicator"] = 0
+            data.loc[
+                data[experiment_group_column] == treatment_group, "treatment_indicator"
+            ] = 1
+            all_result = self.filter_LR(
+                data=data,
+                disp=True,
+                treatment_indicator="treatment_indicator",
+                features=features,
+                y_name=y_name,
             )
         else:
-            all_result = self.filter_D(data=data, method=method,
-                features=features, y_name=y_name, 
-                n_bins=n_bins, control_group=control_group,
-                experiment_group_column=experiment_group_column, 
-                null_impute=null_impute
+            all_result = self.filter_D(
+                data=data,
+                method=method,
+                features=features,
+                y_name=y_name,
+                n_bins=n_bins,
+                control_group=control_group,
+                experiment_group_column=experiment_group_column,
+                null_impute=null_impute,
             )
-        
-        all_result['method'] = method + ' filter'
-        return all_result[['method', 'feature', 'rank', 'score', 'p_value', 'misc']]
+
+        all_result["method"] = method + " filter"
+        return all_result[["method", "feature", "rank", "score", "p_value", "misc"]]
