@@ -1,5 +1,11 @@
 from .slearner import LRSRegressor, BaseSLearner, BaseSRegressor, BaseSClassifier
-from .tlearner import XGBTRegressor, MLPTRegressor, BaseTLearner, BaseTRegressor, BaseTClassifier
+from .tlearner import (
+    XGBTRegressor,
+    MLPTRegressor,
+    BaseTLearner,
+    BaseTRegressor,
+    BaseTClassifier,
+)
 from .xlearner import BaseXLearner, BaseXRegressor, BaseXClassifier
 from .rlearner import BaseRLearner, BaseRRegressor, BaseRClassifier, XGBRRegressor
 from .tmle import TMLELearner
