@@ -6,7 +6,8 @@ from scipy.stats import norm
 import sklearn
 from sklearn.exceptions import ConvergenceWarning
 from sklearn.neural_network import MLPRegressor
-if version.parse(sklearn.__version__) >= version.parse('0.22.0'):
+
+if version.parse(sklearn.__version__) >= version.parse("0.22.0"):
     from sklearn.utils._testing import ignore_warnings
 else:
     from sklearn.utils.testing import ignore_warnings
@@ -19,7 +20,7 @@ from causalml.inference.meta.utils import check_treatment_vector, convert_pd_to_
 from causalml.metrics import regression_metrics, classification_metrics
 
 
-logger = logging.getLogger('causalml')
+logger = logging.getLogger("causalml")
 
 
 class BaseTLearner(BaseLearner):
@@ -30,7 +31,14 @@ class BaseTLearner(BaseLearner):
     Details of T-learner are available at Kunzel et al. (2018) (https://arxiv.org/abs/1706.03461).
     """
 
-    def __init__(self, learner=None, control_learner=None, treatment_learner=None, ate_alpha=.05, control_name=0):
+    def __init__(
+        self,
+        learner=None,
+        control_learner=None,
+        treatment_learner=None,
+        ate_alpha=0.05,
+        control_name=0,
+    ):
         """Initialize a T-learner.
 
         Args:
@@ -40,7 +48,9 @@ class BaseTLearner(BaseLearner):
             ate_alpha (float, optional): the confidence level alpha of the ATE estimate
             control_name (str or int, optional): name of control group
         """
-        assert (learner is not None) or ((control_learner is not None) and (treatment_learner is not None))
+        assert (learner is not None) or (
+            (control_learner is not None) and (treatment_learner is not None)
+        )
 
         if control_learner is None:
             self.model_c = deepcopy(learner)
@@ -56,9 +66,9 @@ class BaseTLearner(BaseLearner):
         self.control_name = control_name
 
     def __repr__(self):
-        return '{}(model_c={}, model_t={})'.format(self.__class__.__name__,
-                                                   self.model_c.__repr__(),
-                                                   self.model_t.__repr__())
+        return "{}(model_c={}, model_t={})".format(
+            self.__class__.__name__, self.model_c.__repr__(), self.model_t.__repr__()
+        )
 
     @ignore_warnings(category=ConvergenceWarning)
     def fit(self, X, treatment, y, p=None):
@@ -87,7 +97,9 @@ class BaseTLearner(BaseLearner):
             self.models_c[group].fit(X_filt[w == 0], y_filt[w == 0])
             self.models_t[group].fit(X_filt[w == 1], y_filt[w == 1])
 
-    def predict(self, X, treatment=None, y=None, p=None, return_components=False, verbose=True):
+    def predict(
+        self, X, treatment=None, y=None, p=None, return_components=False, verbose=True
+    ):
         """Predict treatment effects.
 
         Args:
@@ -119,7 +131,7 @@ class BaseTLearner(BaseLearner):
                 yhat[w == 0] = yhat_cs[group][mask][w == 0]
                 yhat[w == 1] = yhat_ts[group][mask][w == 1]
 
-                logger.info('Error metrics for group {}'.format(group))
+                logger.info("Error metrics for group {}".format(group))
                 regression_metrics(y_filt, yhat, w)
 
         te = np.zeros((X.shape[0], self.t_groups.shape[0]))
@@ -131,8 +143,18 @@ class BaseTLearner(BaseLearner):
         else:
             return te, yhat_cs, yhat_ts
 
-    def fit_predict(self, X, treatment, y, p=None, return_ci=False, n_bootstraps=1000, bootstrap_size=10000,
-                    return_components=False, verbose=True):
+    def fit_predict(
+        self,
+        X,
+        treatment,
+        y,
+        p=None,
+        return_ci=False,
+        n_bootstraps=1000,
+        bootstrap_size=10000,
+        return_components=False,
+        verbose=True,
+    ):
         """Fit the inference model of the T learner and predict treatment effects.
 
         Args:
@@ -160,15 +182,19 @@ class BaseTLearner(BaseLearner):
             _classes_global = self._classes
             models_c_global = deepcopy(self.models_c)
             models_t_global = deepcopy(self.models_t)
-            te_bootstraps = np.zeros(shape=(X.shape[0], self.t_groups.shape[0], n_bootstraps))
+            te_bootstraps = np.zeros(
+                shape=(X.shape[0], self.t_groups.shape[0], n_bootstraps)
+            )
 
-            logger.info('Bootstrap Confidence Intervals')
+            logger.info("Bootstrap Confidence Intervals")
             for i in tqdm(range(n_bootstraps)):
                 te_b = self.bootstrap(X, treatment, y, size=bootstrap_size)
                 te_bootstraps[:, :, i] = te_b
 
-            te_lower = np.percentile(te_bootstraps, (self.ate_alpha/2)*100, axis=2)
-            te_upper = np.percentile(te_bootstraps, (1 - self.ate_alpha / 2) * 100, axis=2)
+            te_lower = np.percentile(te_bootstraps, (self.ate_alpha / 2) * 100, axis=2)
+            te_upper = np.percentile(
+                te_bootstraps, (1 - self.ate_alpha / 2) * 100, axis=2
+            )
 
             # set member variables back to global (currently last bootstrapped outcome)
             self.t_groups = t_groups_global
@@ -178,7 +204,16 @@ class BaseTLearner(BaseLearner):
 
             return (te, te_lower, te_upper)
 
-    def estimate_ate(self, X, treatment, y, p=None, bootstrap_ci=False, n_bootstraps=1000, bootstrap_size=10000):
+    def estimate_ate(
+        self,
+        X,
+        treatment,
+        y,
+        p=None,
+        bootstrap_ci=False,
+        n_bootstraps=1000,
+        bootstrap_size=10000,
+    ):
         """Estimate the Average Treatment Effect (ATE).
 
         Args:
@@ -210,13 +245,14 @@ class BaseTLearner(BaseLearner):
             yhat_c = yhat_cs[group][mask]
             yhat_t = yhat_ts[group][mask]
 
-            se = np.sqrt((
-                (y_filt[w == 0] - yhat_c[w == 0]).var()
-                / (1 - prob_treatment) +
-                (y_filt[w == 1] - yhat_t[w == 1]).var()
-                / prob_treatment +
-                (yhat_t - yhat_c).var()
-            ) / y_filt.shape[0])
+            se = np.sqrt(
+                (
+                    (y_filt[w == 0] - yhat_c[w == 0]).var() / (1 - prob_treatment)
+                    + (y_filt[w == 1] - yhat_t[w == 1]).var() / prob_treatment
+                    + (yhat_t - yhat_c).var()
+                )
+                / y_filt.shape[0]
+            )
 
             _ate_lb = _ate - se * norm.ppf(1 - self.ate_alpha / 2)
             _ate_ub = _ate + se * norm.ppf(1 - self.ate_alpha / 2)
@@ -233,15 +269,19 @@ class BaseTLearner(BaseLearner):
             models_c_global = deepcopy(self.models_c)
             models_t_global = deepcopy(self.models_t)
 
-            logger.info('Bootstrap Confidence Intervals for ATE')
+            logger.info("Bootstrap Confidence Intervals for ATE")
             ate_bootstraps = np.zeros(shape=(self.t_groups.shape[0], n_bootstraps))
 
             for n in tqdm(range(n_bootstraps)):
                 ate_b = self.bootstrap(X, treatment, y, size=bootstrap_size)
                 ate_bootstraps[:, n] = ate_b.mean()
 
-            ate_lower = np.percentile(ate_bootstraps, (self.ate_alpha / 2) * 100, axis=1)
-            ate_upper = np.percentile(ate_bootstraps, (1 - self.ate_alpha / 2) * 100, axis=1)
+            ate_lower = np.percentile(
+                ate_bootstraps, (self.ate_alpha / 2) * 100, axis=1
+            )
+            ate_upper = np.percentile(
+                ate_bootstraps, (1 - self.ate_alpha / 2) * 100, axis=1
+            )
 
             # set member variables back to global (currently last bootstrapped outcome)
             self.t_groups = t_groups_global
@@ -257,12 +297,14 @@ class BaseTRegressor(BaseTLearner):
     A parent class for T-learner regressor classes.
     """
 
-    def __init__(self,
-                 learner=None,
-                 control_learner=None,
-                 treatment_learner=None,
-                 ate_alpha=.05,
-                 control_name=0):
+    def __init__(
+        self,
+        learner=None,
+        control_learner=None,
+        treatment_learner=None,
+        ate_alpha=0.05,
+        control_name=0,
+    ):
         """Initialize a T-learner regressor.
 
         Args:
@@ -277,7 +319,8 @@ class BaseTRegressor(BaseTLearner):
             control_learner=control_learner,
             treatment_learner=treatment_learner,
             ate_alpha=ate_alpha,
-            control_name=control_name)
+            control_name=control_name,
+        )
 
 
 class BaseTClassifier(BaseTLearner):
@@ -285,12 +328,14 @@ class BaseTClassifier(BaseTLearner):
     A parent class for T-learner classifier classes.
     """
 
-    def __init__(self,
-                 learner=None,
-                 control_learner=None,
-                 treatment_learner=None,
-                 ate_alpha=.05,
-                 control_name=0):
+    def __init__(
+        self,
+        learner=None,
+        control_learner=None,
+        treatment_learner=None,
+        ate_alpha=0.05,
+        control_name=0,
+    ):
         """Initialize a T-learner classifier.
 
         Args:
@@ -305,9 +350,12 @@ class BaseTClassifier(BaseTLearner):
             control_learner=control_learner,
             treatment_learner=treatment_learner,
             ate_alpha=ate_alpha,
-            control_name=control_name)
+            control_name=control_name,
+        )
 
-    def predict(self, X, treatment=None, y=None, p=None, return_components=False, verbose=True):
+    def predict(
+        self, X, treatment=None, y=None, p=None, return_components=False, verbose=True
+    ):
         """Predict treatment effects.
 
         Args:
@@ -337,7 +385,7 @@ class BaseTClassifier(BaseTLearner):
                 yhat[w == 0] = yhat_cs[group][mask][w == 0]
                 yhat[w == 1] = yhat_ts[group][mask][w == 1]
 
-                logger.info('Error metrics for group {}'.format(group))
+                logger.info("Error metrics for group {}".format(group))
                 classification_metrics(y_filt, yhat, w)
 
         te = np.zeros((X.shape[0], self.t_groups.shape[0]))
@@ -351,16 +399,20 @@ class BaseTClassifier(BaseTLearner):
 
 
 class XGBTRegressor(BaseTRegressor):
-    def __init__(self, ate_alpha=.05, control_name=0, *args, **kwargs):
+    def __init__(self, ate_alpha=0.05, control_name=0, *args, **kwargs):
         """Initialize a T-learner with two XGBoost models."""
-        super().__init__(learner=XGBRegressor(*args, **kwargs),
-                         ate_alpha=ate_alpha,
-                         control_name=control_name)
+        super().__init__(
+            learner=XGBRegressor(*args, **kwargs),
+            ate_alpha=ate_alpha,
+            control_name=control_name,
+        )
 
 
 class MLPTRegressor(BaseTRegressor):
-    def __init__(self, ate_alpha=.05, control_name=0, *args, **kwargs):
+    def __init__(self, ate_alpha=0.05, control_name=0, *args, **kwargs):
         """Initialize a T-learner with two MLP models."""
-        super().__init__(learner=MLPRegressor(*args, **kwargs),
-                         ate_alpha=ate_alpha,
-                         control_name=control_name)
+        super().__init__(
+            learner=MLPRegressor(*args, **kwargs),
+            ate_alpha=ate_alpha,
+            control_name=control_name,
+        )
