@@ -6,12 +6,16 @@ from tqdm import tqdm
 from scipy.stats import norm
 
 from causalml.inference.meta.base import BaseLearner
-from causalml.inference.meta.utils import check_treatment_vector, check_p_conditions, convert_pd_to_np
+from causalml.inference.meta.utils import (
+    check_treatment_vector,
+    check_p_conditions,
+    convert_pd_to_np,
+)
 from causalml.inference.meta.explainer import Explainer
 from causalml.metrics import regression_metrics, classification_metrics
 from causalml.propensity import compute_propensity_score
 
-logger = logging.getLogger('causalml')
+logger = logging.getLogger("causalml")
 
 
 class BaseXLearner(BaseLearner):
@@ -22,14 +26,16 @@ class BaseXLearner(BaseLearner):
     Details of X-learner are available at Kunzel et al. (2018) (https://arxiv.org/abs/1706.03461).
     """
 
-    def __init__(self,
-                 learner=None,
-                 control_outcome_learner=None,
-                 treatment_outcome_learner=None,
-                 control_effect_learner=None,
-                 treatment_effect_learner=None,
-                 ate_alpha=.05,
-                 control_name=0):
+    def __init__(
+        self,
+        learner=None,
+        control_outcome_learner=None,
+        treatment_outcome_learner=None,
+        control_effect_learner=None,
+        treatment_effect_learner=None,
+        ate_alpha=0.05,
+        control_name=0,
+    ):
         """Initialize a X-learner.
 
         Args:
@@ -42,10 +48,12 @@ class BaseXLearner(BaseLearner):
             ate_alpha (float, optional): the confidence level alpha of the ATE estimate
             control_name (str or int, optional): name of control group
         """
-        assert (learner is not None) or ((control_outcome_learner is not None) and
-                                         (treatment_outcome_learner is not None) and
-                                         (control_effect_learner is not None) and
-                                         (treatment_effect_learner is not None))
+        assert (learner is not None) or (
+            (control_outcome_learner is not None)
+            and (treatment_outcome_learner is not None)
+            and (control_effect_learner is not None)
+            and (treatment_effect_learner is not None)
+        )
 
         if control_outcome_learner is None:
             self.model_mu_c = deepcopy(learner)
@@ -74,14 +82,18 @@ class BaseXLearner(BaseLearner):
         self.propensity_model = None
 
     def __repr__(self):
-        return ('{}(control_outcome_learner={},\n'
-                '\ttreatment_outcome_learner={},\n'
-                '\tcontrol_effect_learner={},\n'
-                '\ttreatment_effect_learner={})'.format(self.__class__.__name__,
-                                                        self.model_mu_c.__repr__(),
-                                                        self.model_mu_t.__repr__(),
-                                                        self.model_tau_c.__repr__(),
-                                                        self.model_tau_t.__repr__()))
+        return (
+            "{}(control_outcome_learner={},\n"
+            "\ttreatment_outcome_learner={},\n"
+            "\tcontrol_effect_learner={},\n"
+            "\ttreatment_effect_learner={})".format(
+                self.__class__.__name__,
+                self.model_mu_c.__repr__(),
+                self.model_mu_t.__repr__(),
+                self.model_tau_c.__repr__(),
+                self.model_tau_t.__repr__(),
+            )
+        )
 
     def fit(self, X, treatment, y, p=None):
         """Fit the inference model.
@@ -108,8 +120,12 @@ class BaseXLearner(BaseLearner):
         self._classes = {group: i for i, group in enumerate(self.t_groups)}
         self.models_mu_c = {group: deepcopy(self.model_mu_c) for group in self.t_groups}
         self.models_mu_t = {group: deepcopy(self.model_mu_t) for group in self.t_groups}
-        self.models_tau_c = {group: deepcopy(self.model_tau_c) for group in self.t_groups}
-        self.models_tau_t = {group: deepcopy(self.model_tau_t) for group in self.t_groups}
+        self.models_tau_c = {
+            group: deepcopy(self.model_tau_c) for group in self.t_groups
+        }
+        self.models_tau_t = {
+            group: deepcopy(self.model_tau_t) for group in self.t_groups
+        }
         self.vars_c = {}
         self.vars_t = {}
 
@@ -125,9 +141,13 @@ class BaseXLearner(BaseLearner):
             self.models_mu_t[group].fit(X_filt[w == 1], y_filt[w == 1])
 
             # Calculate variances and treatment effects
-            var_c = (y_filt[w == 0] - self.models_mu_c[group].predict(X_filt[w == 0])).var()
+            var_c = (
+                y_filt[w == 0] - self.models_mu_c[group].predict(X_filt[w == 0])
+            ).var()
             self.vars_c[group] = var_c
-            var_t = (y_filt[w == 1] - self.models_mu_t[group].predict(X_filt[w == 1])).var()
+            var_t = (
+                y_filt[w == 1] - self.models_mu_t[group].predict(X_filt[w == 1])
+            ).var()
             self.vars_t[group] = var_t
 
             # Train treatment models
@@ -136,8 +156,9 @@ class BaseXLearner(BaseLearner):
             self.models_tau_c[group].fit(X_filt[w == 0], d_c)
             self.models_tau_t[group].fit(X_filt[w == 1], d_t)
 
-    def predict(self, X, treatment=None, y=None, p=None, return_components=False,
-                verbose=True):
+    def predict(
+        self, X, treatment=None, y=None, p=None, return_components=False, verbose=True
+    ):
         """Predict treatment effects.
 
         Args:
@@ -155,7 +176,7 @@ class BaseXLearner(BaseLearner):
         X, treatment, y = convert_pd_to_np(X, treatment, y)
 
         if p is None:
-            logger.info('Generating propensity score')
+            logger.info("Generating propensity score")
             p = dict()
             for group in self.t_groups:
                 p_model = self.propensity_model[group]
@@ -173,7 +194,9 @@ class BaseXLearner(BaseLearner):
             dhat_cs[group] = model_tau_c.predict(X)
             dhat_ts[group] = model_tau_t.predict(X)
 
-            _te = (p[group] * dhat_cs[group] + (1 - p[group]) * dhat_ts[group]).reshape(-1, 1)
+            _te = (p[group] * dhat_cs[group] + (1 - p[group]) * dhat_ts[group]).reshape(
+                -1, 1
+            )
             te[:, i] = np.ravel(_te)
 
             if (y is not None) and (treatment is not None) and verbose:
@@ -187,7 +210,7 @@ class BaseXLearner(BaseLearner):
                 yhat[w == 0] = self.models_mu_c[group].predict(X_filt[w == 0])
                 yhat[w == 1] = self.models_mu_t[group].predict(X_filt[w == 1])
 
-                logger.info('Error metrics for group {}'.format(group))
+                logger.info("Error metrics for group {}".format(group))
                 regression_metrics(y_filt, yhat, w)
 
         if not return_components:
@@ -195,8 +218,18 @@ class BaseXLearner(BaseLearner):
         else:
             return te, dhat_cs, dhat_ts
 
-    def fit_predict(self, X, treatment, y, p=None, return_ci=False, n_bootstraps=1000, bootstrap_size=10000,
-                    return_components=False, verbose=True):
+    def fit_predict(
+        self,
+        X,
+        treatment,
+        y,
+        p=None,
+        return_ci=False,
+        n_bootstraps=1000,
+        bootstrap_size=10000,
+        return_components=False,
+        verbose=True,
+    ):
         """Fit the treatment effect and outcome models of the R learner and predict treatment effects.
 
         Args:
@@ -224,7 +257,9 @@ class BaseXLearner(BaseLearner):
         else:
             p = self._format_p(p, self.t_groups)
 
-        te = self.predict(X, treatment=treatment, y=y, p=p, return_components=return_components)
+        te = self.predict(
+            X, treatment=treatment, y=y, p=p, return_components=return_components
+        )
 
         if not return_ci:
             return te
@@ -235,15 +270,19 @@ class BaseXLearner(BaseLearner):
             models_mu_t_global = deepcopy(self.models_mu_t)
             models_tau_c_global = deepcopy(self.models_tau_c)
             models_tau_t_global = deepcopy(self.models_tau_t)
-            te_bootstraps = np.zeros(shape=(X.shape[0], self.t_groups.shape[0], n_bootstraps))
+            te_bootstraps = np.zeros(
+                shape=(X.shape[0], self.t_groups.shape[0], n_bootstraps)
+            )
 
-            logger.info('Bootstrap Confidence Intervals')
+            logger.info("Bootstrap Confidence Intervals")
             for i in tqdm(range(n_bootstraps)):
                 te_b = self.bootstrap(X, treatment, y, p, size=bootstrap_size)
                 te_bootstraps[:, :, i] = te_b
 
             te_lower = np.percentile(te_bootstraps, (self.ate_alpha / 2) * 100, axis=2)
-            te_upper = np.percentile(te_bootstraps, (1 - self.ate_alpha / 2) * 100, axis=2)
+            te_upper = np.percentile(
+                te_bootstraps, (1 - self.ate_alpha / 2) * 100, axis=2
+            )
 
             # set member variables back to global (currently last bootstrapped outcome)
             self.t_groups = t_groups_global
@@ -255,7 +294,16 @@ class BaseXLearner(BaseLearner):
 
             return (te, te_lower, te_upper)
 
-    def estimate_ate(self, X, treatment, y, p=None, bootstrap_ci=False, n_bootstraps=1000, bootstrap_size=10000):
+    def estimate_ate(
+        self,
+        X,
+        treatment,
+        y,
+        p=None,
+        bootstrap_ci=False,
+        n_bootstraps=1000,
+        bootstrap_size=10000,
+    ):
         """Estimate the Average Treatment Effect (ATE).
 
         Args:
@@ -271,7 +319,9 @@ class BaseXLearner(BaseLearner):
         Returns:
             The mean and confidence interval (LB, UB) of the ATE estimate.
         """
-        te, dhat_cs, dhat_ts = self.fit_predict(X, treatment, y, p, return_components=True)
+        te, dhat_cs, dhat_ts = self.fit_predict(
+            X, treatment, y, p, return_components=True
+        )
         X, treatment, y = convert_pd_to_np(X, treatment, y)
 
         if p is None:
@@ -297,10 +347,14 @@ class BaseXLearner(BaseLearner):
 
             # SE formula is based on the lower bound formula (7) from Imbens, Guido W., and Jeffrey M. Wooldridge. 2009.
             # "Recent Developments in the Econometrics of Program Evaluation." Journal of Economic Literature
-            se = np.sqrt((
-                self.vars_t[group] / prob_treatment + self.vars_c[group] / (1 - prob_treatment) +
-                (p_filt * dhat_c + (1 - p_filt) * dhat_t).var()
-            ) / w.shape[0])
+            se = np.sqrt(
+                (
+                    self.vars_t[group] / prob_treatment
+                    + self.vars_c[group] / (1 - prob_treatment)
+                    + (p_filt * dhat_c + (1 - p_filt) * dhat_t).var()
+                )
+                / w.shape[0]
+            )
 
             _ate_lb = _ate - se * norm.ppf(1 - self.ate_alpha / 2)
             _ate_ub = _ate + se * norm.ppf(1 - self.ate_alpha / 2)
@@ -319,15 +373,19 @@ class BaseXLearner(BaseLearner):
             models_tau_c_global = deepcopy(self.models_tau_c)
             models_tau_t_global = deepcopy(self.models_tau_t)
 
-            logger.info('Bootstrap Confidence Intervals for ATE')
+            logger.info("Bootstrap Confidence Intervals for ATE")
             ate_bootstraps = np.zeros(shape=(self.t_groups.shape[0], n_bootstraps))
 
             for n in tqdm(range(n_bootstraps)):
                 cate_b = self.bootstrap(X, treatment, y, p, size=bootstrap_size)
                 ate_bootstraps[:, n] = cate_b.mean()
 
-            ate_lower = np.percentile(ate_bootstraps, (self.ate_alpha / 2) * 100, axis=1)
-            ate_upper = np.percentile(ate_bootstraps, (1 - self.ate_alpha / 2) * 100, axis=1)
+            ate_lower = np.percentile(
+                ate_bootstraps, (self.ate_alpha / 2) * 100, axis=1
+            )
+            ate_upper = np.percentile(
+                ate_bootstraps, (1 - self.ate_alpha / 2) * 100, axis=1
+            )
 
             # set member variables back to global (currently last bootstrapped outcome)
             self.t_groups = t_groups_global
@@ -344,14 +402,16 @@ class BaseXRegressor(BaseXLearner):
     A parent class for X-learner regressor classes.
     """
 
-    def __init__(self,
-                 learner=None,
-                 control_outcome_learner=None,
-                 treatment_outcome_learner=None,
-                 control_effect_learner=None,
-                 treatment_effect_learner=None,
-                 ate_alpha=.05,
-                 control_name=0):
+    def __init__(
+        self,
+        learner=None,
+        control_outcome_learner=None,
+        treatment_outcome_learner=None,
+        control_effect_learner=None,
+        treatment_effect_learner=None,
+        ate_alpha=0.05,
+        control_name=0,
+    ):
         """Initialize an X-learner regressor.
 
         Args:
@@ -371,7 +431,8 @@ class BaseXRegressor(BaseXLearner):
             control_effect_learner=control_effect_learner,
             treatment_effect_learner=treatment_effect_learner,
             ate_alpha=ate_alpha,
-            control_name=control_name)
+            control_name=control_name,
+        )
 
 
 class BaseXClassifier(BaseXLearner):
@@ -379,15 +440,17 @@ class BaseXClassifier(BaseXLearner):
     A parent class for X-learner classifier classes.
     """
 
-    def __init__(self,
-                 outcome_learner=None,
-                 effect_learner=None,
-                 control_outcome_learner=None,
-                 treatment_outcome_learner=None,
-                 control_effect_learner=None,
-                 treatment_effect_learner=None,
-                 ate_alpha=.05,
-                 control_name=0):
+    def __init__(
+        self,
+        outcome_learner=None,
+        effect_learner=None,
+        control_outcome_learner=None,
+        treatment_outcome_learner=None,
+        control_effect_learner=None,
+        treatment_effect_learner=None,
+        ate_alpha=0.05,
+        control_name=0,
+    ):
         """Initialize an X-learner classifier.
 
         Args:
@@ -420,11 +483,15 @@ class BaseXClassifier(BaseXLearner):
             control_effect_learner=control_effect_learner,
             treatment_effect_learner=treatment_effect_learner,
             ate_alpha=ate_alpha,
-            control_name=control_name)
+            control_name=control_name,
+        )
 
-        if ((control_outcome_learner is None) or (treatment_outcome_learner is None)) and (
-                (control_effect_learner is None) or (treatment_effect_learner is None)):
-            raise ValueError("Either the outcome learner or the effect learner pair must be specified.")
+        if (
+            (control_outcome_learner is None) or (treatment_outcome_learner is None)
+        ) and ((control_effect_learner is None) or (treatment_effect_learner is None)):
+            raise ValueError(
+                "Either the outcome learner or the effect learner pair must be specified."
+            )
 
     def fit(self, X, treatment, y, p=None):
         """Fit the inference model.
@@ -451,8 +518,12 @@ class BaseXClassifier(BaseXLearner):
         self._classes = {group: i for i, group in enumerate(self.t_groups)}
         self.models_mu_c = {group: deepcopy(self.model_mu_c) for group in self.t_groups}
         self.models_mu_t = {group: deepcopy(self.model_mu_t) for group in self.t_groups}
-        self.models_tau_c = {group: deepcopy(self.model_tau_c) for group in self.t_groups}
-        self.models_tau_t = {group: deepcopy(self.model_tau_t) for group in self.t_groups}
+        self.models_tau_c = {
+            group: deepcopy(self.model_tau_c) for group in self.t_groups
+        }
+        self.models_tau_t = {
+            group: deepcopy(self.model_tau_t) for group in self.t_groups
+        }
         self.vars_c = {}
         self.vars_t = {}
 
@@ -468,19 +539,32 @@ class BaseXClassifier(BaseXLearner):
             self.models_mu_t[group].fit(X_filt[w == 1], y_filt[w == 1])
 
             # Calculate variances and treatment effects
-            var_c = (y_filt[w == 0] - self.models_mu_c[group].predict_proba(X_filt[w == 0])[:, 1]).var()
+            var_c = (
+                y_filt[w == 0]
+                - self.models_mu_c[group].predict_proba(X_filt[w == 0])[:, 1]
+            ).var()
             self.vars_c[group] = var_c
-            var_t = (y_filt[w == 1] - self.models_mu_t[group].predict_proba(X_filt[w == 1])[:, 1]).var()
+            var_t = (
+                y_filt[w == 1]
+                - self.models_mu_t[group].predict_proba(X_filt[w == 1])[:, 1]
+            ).var()
             self.vars_t[group] = var_t
 
             # Train treatment models
-            d_c = self.models_mu_t[group].predict_proba(X_filt[w == 0])[:, 1] - y_filt[w == 0]
-            d_t = y_filt[w == 1] - self.models_mu_c[group].predict_proba(X_filt[w == 1])[:, 1]
+            d_c = (
+                self.models_mu_t[group].predict_proba(X_filt[w == 0])[:, 1]
+                - y_filt[w == 0]
+            )
+            d_t = (
+                y_filt[w == 1]
+                - self.models_mu_c[group].predict_proba(X_filt[w == 1])[:, 1]
+            )
             self.models_tau_c[group].fit(X_filt[w == 0], d_c)
             self.models_tau_t[group].fit(X_filt[w == 1], d_t)
 
-    def predict(self, X, treatment=None, y=None, p=None, return_components=False,
-                verbose=True):
+    def predict(
+        self, X, treatment=None, y=None, p=None, return_components=False, verbose=True
+    ):
         """Predict treatment effects.
 
         Args:
@@ -499,7 +583,7 @@ class BaseXClassifier(BaseXLearner):
         X, treatment, y = convert_pd_to_np(X, treatment, y)
 
         if p is None:
-            logger.info('Generating propensity score')
+            logger.info("Generating propensity score")
             p = dict()
             for group in self.t_groups:
                 p_model = self.propensity_model[group]
@@ -517,7 +601,9 @@ class BaseXClassifier(BaseXLearner):
             dhat_cs[group] = model_tau_c.predict(X)
             dhat_ts[group] = model_tau_t.predict(X)
 
-            _te = (p[group] * dhat_cs[group] + (1 - p[group]) * dhat_ts[group]).reshape(-1, 1)
+            _te = (p[group] * dhat_cs[group] + (1 - p[group]) * dhat_ts[group]).reshape(
+                -1, 1
+            )
             te[:, i] = np.ravel(_te)
 
             if (y is not None) and (treatment is not None) and verbose:
@@ -528,10 +614,14 @@ class BaseXClassifier(BaseXLearner):
                 w = (treatment_filt == group).astype(int)
 
                 yhat = np.zeros_like(y_filt, dtype=float)
-                yhat[w == 0] = self.models_mu_c[group].predict_proba(X_filt[w == 0])[:, 1]
-                yhat[w == 1] = self.models_mu_t[group].predict_proba(X_filt[w == 1])[:, 1]
+                yhat[w == 0] = self.models_mu_c[group].predict_proba(X_filt[w == 0])[
+                    :, 1
+                ]
+                yhat[w == 1] = self.models_mu_t[group].predict_proba(X_filt[w == 1])[
+                    :, 1
+                ]
 
-                logger.info('Error metrics for group {}'.format(group))
+                logger.info("Error metrics for group {}".format(group))
                 classification_metrics(y_filt, yhat, w)
 
         if not return_components:
