@@ -7,7 +7,7 @@ from causalml.dataset import make_uplift_classification
 from .const import RANDOM_SEED, N_SAMPLE, TREATMENT_NAMES, CONVERSION
 
 
-@pytest.fixture(scope='module')
+@pytest.fixture(scope="module")
 def generate_regression_data():
 
     generated = False
@@ -15,14 +15,14 @@ def generate_regression_data():
     def _generate_data():
         if not generated:
             np.random.seed(RANDOM_SEED)
-            data = synthetic_data(mode=1, n=N_SAMPLE, p=8, sigma=.1)
+            data = synthetic_data(mode=1, n=N_SAMPLE, p=8, sigma=0.1)
 
         return data
 
     yield _generate_data
 
 
-@pytest.fixture(scope='module')
+@pytest.fixture(scope="module")
 def generate_classification_data():
 
     generated = False
@@ -30,16 +30,19 @@ def generate_classification_data():
     def _generate_data():
         if not generated:
             np.random.seed(RANDOM_SEED)
-            data = make_uplift_classification(n_samples=N_SAMPLE,
-                                              treatment_name=TREATMENT_NAMES,
-                                              y_name=CONVERSION,
-                                              random_seed=RANDOM_SEED)
+            data = make_uplift_classification(
+                n_samples=N_SAMPLE,
+                treatment_name=TREATMENT_NAMES,
+                y_name=CONVERSION,
+                random_seed=RANDOM_SEED,
+            )
 
         return data
 
     yield _generate_data
 
-@pytest.fixture(scope='module')
+
+@pytest.fixture(scope="module")
 def generate_classification_data_two_treatments():
 
     generated = False
@@ -47,10 +50,12 @@ def generate_classification_data_two_treatments():
     def _generate_data():
         if not generated:
             np.random.seed(RANDOM_SEED)
-            data = make_uplift_classification(n_samples=N_SAMPLE,
-                                              treatment_name=TREATMENT_NAMES[0:2],
-                                              y_name=CONVERSION,
-                                              random_seed=RANDOM_SEED)
+            data = make_uplift_classification(
+                n_samples=N_SAMPLE,
+                treatment_name=TREATMENT_NAMES[0:2],
+                y_name=CONVERSION,
+                random_seed=RANDOM_SEED,
+            )
 
         return data
 
